"""
AWS CDK Secure API - A CDK (v2) Construct Library for Secure REST APIs
"""

__title__ = 'aws-cdk-secure-api'
__description__ = 'A CDK (v2) Construct Library for Secure REST APIs'
__url__ = 'https://github.com/rnag/aws-cdk-secure-api'
__version__ = '0.1.1'
__author__ = 'Ritvik Nag'
__author_email__ = 'rv.kvetch@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022 Ritvik Nag'
